@@ -36,6 +36,10 @@ the AWS_UNSTABLE_TESTING_API compiler flag
 #    pragma warning(disable : 4204) /* non-constant aggregate initializer */
 #endif
 
+#if defined(__clang__)
+#    pragma clang diagnostic ignored "-Wgnu-zero-variadic-macro-arguments"
+#endif
+
 /** Prints a message to AWS_TESTING_REPORT_FD using printf format that appends the function, file and line number.
  * If format is null, returns 0 without printing anything; otherwise returns 1.
  */
@@ -71,9 +75,11 @@ static int total_skip;
 
 #define SUCCESS (0)
 #define FAILURE (-1)
-/* The skip code returned to ctest to indicate the test is skipped. Refer to cmake doc:
- * https://cmake.org/cmake/help/latest/prop_test/SKIP_RETURN_CODE.html */
-#define SKIP (1)
+/* The exit code returned to ctest to indicate the test is skipped. Refer to cmake doc:
+ * https://cmake.org/cmake/help/latest/prop_test/SKIP_RETURN_CODE.html
+ * The value has no special meaning, it's just an arbitrary exit code reducing the chance of clashing with exit codes
+ * that may be returned from various tools (e.g. sanitizer). */
+#define SKIP (103)
 
 #define POSTSKIP_INTERNAL()                                                                                            \
     do {                                                                                                               \
@@ -94,11 +100,11 @@ static int total_skip;
         printf("\n");                                                                                                  \
         return SUCCESS;                                                                                                \
     } while (0)
-#define PRINT_FAIL_INTERNAL(...)                                                                                       \
-    CUNIT_FAILURE_MESSAGE(__FUNCTION__, __FILE__, __LINE__, ##__VA_ARGS__, (const char *)NULL)
+
+#define PRINT_FAIL_INTERNAL(...) CUNIT_FAILURE_MESSAGE(__func__, __FILE__, __LINE__, ##__VA_ARGS__, (const char *)NULL)
 
 #define PRINT_FAIL_INTERNAL0(...)                                                                                      \
-    s_cunit_failure_message0(FAIL_PREFIX, __FUNCTION__, __FILE__, __LINE__, ##__VA_ARGS__, (const char *)NULL)
+    s_cunit_failure_message0(FAIL_PREFIX, __func__, __FILE__, __LINE__, ##__VA_ARGS__, (const char *)NULL)
 
 #define POSTFAIL_INTERNAL()                                                                                            \
     do {                                                                                                               \
@@ -333,29 +339,25 @@ static int total_skip;
             break;                                                                                                     \
         }                                                                                                              \
         if (assert_ex_s != assert_got_s) {                                                                             \
-            fprintf(AWS_TESTING_REPORT_FD, "%sSize mismatch: %zu != %zu: ", FAIL_PREFIX, assert_ex_s, assert_got_s);   \
+            fprintf(AWS_TESTING_REPORT_FD, "%sSize mismatch: %zu != %zu: \n", FAIL_PREFIX, assert_ex_s, assert_got_s); \
+            fprintf(                                                                                                   \
+                AWS_TESTING_REPORT_FD,                                                                                 \
+                "%sGot: \"" PRInSTR "\"; Expected: \"%s\" \n",                                                         \
+                FAIL_PREFIX,                                                                                           \
+                AWS_BYTE_CURSOR_PRI(cursor),                                                                           \
+                cstring);                                                                                              \
             if (!PRINT_FAIL_INTERNAL0(__VA_ARGS__)) {                                                                  \
                 PRINT_FAIL_INTERNAL0("ASSERT_CURSOR_VALUE_STRING_EQUALS(%s, %s)", #cursor, #cstring);                  \
             }                                                                                                          \
             POSTFAIL_INTERNAL();                                                                                       \
         }                                                                                                              \
         if (memcmp(assert_ex_p, assert_got_p, assert_got_s) != 0) {                                                    \
-            if (assert_got_s <= 1024) {                                                                                \
-                for (size_t assert_i = 0; assert_i < assert_ex_s; ++assert_i) {                                        \
-                    if (assert_ex_p[assert_i] != assert_got_p[assert_i]) {                                             \
-                        fprintf(                                                                                       \
-                            AWS_TESTING_REPORT_FD,                                                                     \
-                            "%sMismatch at byte[%zu]: 0x%02X != 0x%02X: ",                                             \
-                            FAIL_PREFIX,                                                                               \
-                            assert_i,                                                                                  \
-                            assert_ex_p[assert_i],                                                                     \
-                            assert_got_p[assert_i]);                                                                   \
-                        break;                                                                                         \
-                    }                                                                                                  \
-                }                                                                                                      \
-            } else {                                                                                                   \
-                fprintf(AWS_TESTING_REPORT_FD, "%sData mismatch: ", FAIL_PREFIX);                                      \
-            }                                                                                                          \
+            fprintf(                                                                                                   \
+                AWS_TESTING_REPORT_FD,                                                                                 \
+                "%sData mismatch; Got: \"" PRInSTR "\"; Expected: \"%s\" \n",                                          \
+                FAIL_PREFIX,                                                                                           \
+                AWS_BYTE_CURSOR_PRI(cursor),                                                                           \
+                cstring);                                                                                              \
             if (!PRINT_FAIL_INTERNAL0(__VA_ARGS__)) {                                                                  \
                 PRINT_FAIL_INTERNAL0("ASSERT_CURSOR_VALUE_STRING_EQUALS(%s, %s)", #cursor, #cstring);                  \
             }                                                                                                          \
@@ -472,10 +474,11 @@ static inline int s_aws_run_test_case(struct aws_test_harness *harness) {
          * but aws_mem_tracer_dump() needs a valid logger to be active */
         aws_logger_set(&err_logger);
 
-        const size_t leaked_bytes = aws_mem_tracer_count(allocator);
+        const size_t leaked_allocations = aws_mem_tracer_count(allocator);
+        const size_t leaked_bytes = aws_mem_tracer_bytes(allocator);
         if (leaked_bytes) {
             aws_mem_tracer_dump(allocator);
-            PRINT_FAIL_INTERNAL0("Test leaked memory: %zu bytes", leaked_bytes);
+            PRINT_FAIL_INTERNAL0("Test leaked memory: %zu bytes %zu allocations", leaked_bytes, leaked_allocations);
             goto fail;
         }
 
