@@ -470,7 +470,7 @@ static int s_drive_negotiation(struct aws_channel_handler *handler) {
                 "id=%p: Using custom CA, certificate validation failed with OSStatus %d and Trust Eval %d.",
                 (void *)handler,
                 (int)status,
-                (int)trust_eval)
+                (int)trust_eval);
             return AWS_OP_ERR;
         }
         return s_drive_negotiation(handler);
@@ -639,9 +639,8 @@ static int s_process_read_message(
                     (int)status);
 
                 if (status != errSSLClosedGraceful) {
-                    aws_raise_error(AWS_IO_TLS_ERROR_ALERT_RECEIVED);
-                    aws_channel_shutdown(
-                        secure_transport_handler->parent_slot->channel, AWS_IO_TLS_ERROR_ALERT_RECEIVED);
+                    aws_raise_error(AWS_IO_TLS_ERROR_READ_FAILURE);
+                    aws_channel_shutdown(secure_transport_handler->parent_slot->channel, AWS_IO_TLS_ERROR_READ_FAILURE);
                 } else {
                     AWS_LOGF_TRACE(AWS_LS_IO_TLS, "id=%p: connection shutting down gracefully.", (void *)handler);
                     aws_channel_shutdown(secure_transport_handler->parent_slot->channel, AWS_ERROR_SUCCESS);
@@ -1096,28 +1095,4 @@ struct aws_tls_ctx *aws_tls_client_ctx_new(struct aws_allocator *alloc, const st
     return s_tls_ctx_new(alloc, options);
 }
 
-void aws_tls_ctx_destroy(struct aws_tls_ctx *ctx) {
-
-    if (ctx == NULL) {
-        return;
-    }
-
-    struct secure_transport_ctx *secure_transport_ctx = ctx->impl;
-
-    if (secure_transport_ctx->certs) {
-        aws_release_identity(secure_transport_ctx->certs);
-    }
-
-    if (secure_transport_ctx->ca_cert) {
-        aws_release_certificates(secure_transport_ctx->ca_cert);
-    }
-
-    if (secure_transport_ctx->alpn_list) {
-        aws_string_destroy(secure_transport_ctx->alpn_list);
-    }
-
-    CFRelease(secure_transport_ctx->wrapped_allocator);
-    aws_mem_release(secure_transport_ctx->ctx.alloc, secure_transport_ctx);
-}
-
 #pragma clang diagnostic pop
