@@ -86,16 +86,15 @@ enum aws_mqtt5_client_outbound_topic_alias_behavior_type {
      * topic alias mappings unpredictably.  The client will properly use the alias when the current connection
      * has seen the alias binding already.
      */
-    AWS_MQTT5_COTABT_USER,
+    AWS_MQTT5_COTABT_MANUAL,
 
     /**
-     * Client fails any user-specified topic aliasing and acts on the outbound alias set as an LRU cache.
+     * Client ignores any user-specified topic aliasing and acts on the outbound alias set as an LRU cache.
      */
     AWS_MQTT5_COTABT_LRU,
 
     /**
-     * Completely disable outbound topic aliasing.  Attempting to set a topic alias on a PUBLISH results in
-     * an error.
+     * Completely disable outbound topic aliasing.
      */
     AWS_MQTT5_COTABT_DISABLED
 };
@@ -162,7 +161,7 @@ struct aws_mqtt5_client_topic_alias_options {
      * disabled, this setting has no effect.
      *
      * Behaviorally, this value overrides anything present in the topic_alias_maximum field of
-     * the CONNECT packet options.  We intentionally don't bind that field to managed clients to reduce
+     * the CONNECT packet options.
      */
     uint16_t inbound_alias_cache_size;
 };
@@ -344,31 +343,37 @@ typedef void(aws_mqtt5_client_termination_completion_fn)(void *complete_ctx);
 /* operation completion options structures */
 
 /**
- * Completion callback options for the Publish operation
+ * Completion options for the Publish operation
  */
 struct aws_mqtt5_publish_completion_options {
     aws_mqtt5_publish_completion_fn *completion_callback;
     void *completion_user_data;
+
+    uint32_t ack_timeout_seconds_override;
 };
 
 /**
- * Completion callback options for the Subscribe operation
+ * Completion options for the Subscribe operation
  */
 struct aws_mqtt5_subscribe_completion_options {
     aws_mqtt5_subscribe_completion_fn *completion_callback;
     void *completion_user_data;
+
+    uint32_t ack_timeout_seconds_override;
 };
 
 /**
- * Completion callback options for the Unsubscribe operation
+ * Completion options for the Unsubscribe operation
  */
 struct aws_mqtt5_unsubscribe_completion_options {
     aws_mqtt5_unsubscribe_completion_fn *completion_callback;
     void *completion_user_data;
+
+    uint32_t ack_timeout_seconds_override;
 };
 
 /**
- * Public completion callback options for the a DISCONNECT operation
+ * Completion options for the a DISCONNECT operation
  */
 struct aws_mqtt5_disconnect_completion_options {
     aws_mqtt5_disconnect_completion_fn *completion_callback;
