@@ -7,6 +7,7 @@
  */
 
 #include <aws/auth/signing_config.h>
+#include <aws/common/ref_count.h>
 #include <aws/io/retry_strategy.h>
 #include <aws/s3/s3.h>
 
@@ -19,6 +20,7 @@ struct aws_http_message;
 struct aws_http_headers;
 struct aws_tls_connection_options;
 struct aws_input_stream;
+struct aws_hash_table;
 
 struct aws_s3_client;
 struct aws_s3_request;
@@ -29,6 +31,8 @@ struct aws_uri;
 struct aws_string;
 
 struct aws_s3_request_metrics;
+struct aws_s3express_credentials_provider;
+struct aws_credentials_properties_s3express;
 
 /**
  * A Meta Request represents a group of generated requests that are being done on behalf of the
@@ -85,11 +89,21 @@ enum aws_s3_meta_request_type {
 };
 
 /**
- * The type of S3 request made. Used by metrics.
+ * The type of a single S3 HTTP request. Used by metrics.
+ * A meta-request can make multiple S3 HTTP requests under the hood.
+ *
+ * For example, AWS_S3_META_REQUEST_TYPE_PUT_OBJECT for a large file will
+ * do multipart upload, resulting in 3+ HTTP requests:
+ * AWS_S3_REQUEST_TYPE_CREATE_MULTIPART_UPLOAD, one or more AWS_S3_REQUEST_TYPE_UPLOAD_PART,
+ * and finally AWS_S3_REQUEST_TYPE_COMPLETE_MULTIPART_UPLOAD.
+ *
+ * aws_s3_request_type_operation_name() returns the S3 operation name
+ * for types that map (e.g. AWS_S3_REQUEST_TYPE_HEAD_OBJECT -> "HeadObject"),
+ * or empty string for types that don't map (e.g. AWS_S3_REQUEST_TYPE_UNKNOWN -> "").
  */
 enum aws_s3_request_type {
-    /* Same as the original HTTP request passed to aws_s3_meta_request_options */
-    AWS_S3_REQUEST_TYPE_DEFAULT,
+    /* The actual type of the single S3 HTTP request is unknown */
+    AWS_S3_REQUEST_TYPE_UNKNOWN,
 
     /* S3 APIs */
     AWS_S3_REQUEST_TYPE_HEAD_OBJECT,
@@ -100,8 +114,14 @@ enum aws_s3_request_type {
     AWS_S3_REQUEST_TYPE_ABORT_MULTIPART_UPLOAD,
     AWS_S3_REQUEST_TYPE_COMPLETE_MULTIPART_UPLOAD,
     AWS_S3_REQUEST_TYPE_UPLOAD_PART_COPY,
+    AWS_S3_REQUEST_TYPE_COPY_OBJECT,
+    AWS_S3_REQUEST_TYPE_PUT_OBJECT,
 
+    /* Max enum value */
     AWS_S3_REQUEST_TYPE_MAX,
+
+    /** @deprecated Use AWS_S3_REQUEST_TYPE_UNKNOWN if the actual S3 HTTP request type is unknown */
+    AWS_S3_REQUEST_TYPE_DEFAULT = AWS_S3_REQUEST_TYPE_UNKNOWN,
 };
 
 /**
@@ -109,7 +129,10 @@ enum aws_s3_request_type {
  * success and error HTTP status codes.
  *
  * Return AWS_OP_SUCCESS to continue processing the request.
- * Return AWS_OP_ERR to indicate failure and cancel the request.
+ *
+ * Return aws_raise_error(E) to cancel the request.
+ * The error you raise will be reflected in `aws_s3_meta_request_result.error_code`.
+ * If you're not sure which error to raise, use AWS_ERROR_S3_CANCELED.
  */
 typedef int(aws_s3_meta_request_headers_callback_fn)(
     struct aws_s3_meta_request *meta_request,
@@ -133,7 +156,10 @@ typedef int(aws_s3_meta_request_headers_callback_fn)(
  * No back-pressure is applied and data arrives as fast as possible.
  *
  * Return AWS_OP_SUCCESS to continue processing the request.
- * Return AWS_OP_ERR to indicate failure and cancel the request.
+ *
+ * Return aws_raise_error(E) to cancel the request.
+ * The error you raise will be reflected in `aws_s3_meta_request_result.error_code`.
+ * If you're not sure which error to raise, use AWS_ERROR_S3_CANCELED.
  */
 typedef int(aws_s3_meta_request_receive_body_callback_fn)(
 
@@ -171,7 +197,11 @@ struct aws_s3_meta_request_progress {
 };
 
 /**
- * Invoked to report progress of multi-part upload and copy object requests.
+ * Invoked to report progress of a meta-request.
+ * For PutObject, progress refers to bytes uploaded.
+ * For CopyObject, progress refers to bytes copied.
+ * For GetObject, progress refers to bytes downloaded.
+ * For anything else, progress refers to response body bytes received.
  */
 typedef void(aws_s3_meta_request_progress_fn)(
     struct aws_s3_meta_request *meta_request,
@@ -219,6 +249,90 @@ enum aws_s3_checksum_location {
     AWS_SCL_TRAILER,
 };
 
+/**
+ * Info about a single part, for you to review before the upload completes.
+ */
+struct aws_s3_upload_part_review {
+    /* Size in bytes of this part */
+    uint64_t size;
+
+    /* Checksum string, as sent in the UploadPart request (usually base64-encoded):
+     * https://docs.aws.amazon.com/AmazonS3/latest/API/API_UploadPart.html#API_UploadPart_RequestSyntax
+     * This is empty if no checksum is used. */
+    struct aws_byte_cursor checksum;
+};
+
+/**
+ * Info for you to review before an upload completes.
+ *
+ * WARNING: This feature is experimental/unstable.
+ * At this time, review is only available for multipart upload
+ * (when Content-Length is above the `multipart_upload_threshold`,
+ * or Content-Length not specified).
+ */
+struct aws_s3_upload_review {
+    /* The checksum algorithm used. */
+    enum aws_s3_checksum_algorithm checksum_algorithm;
+
+    /* Number of parts uploaded. */
+    size_t part_count;
+
+    /* Array of info about each part uploaded (array is `part_count` in length) */
+    struct aws_s3_upload_part_review *part_array;
+};
+
+/**
+ * Optional callback, for you to review an upload before it completes.
+ * For example, you can review each part's checksum and fail the upload if
+ * you do not agree with them.
+ *
+ * @param meta_request pointer to the aws_s3_meta_request of the upload.
+ * @param info Detailed info about the upload.
+ *
+ * Return AWS_OP_SUCCESS to continue processing the request.
+ *
+ * Return aws_raise_error(E) to cancel the request.
+ * The error you raise will be reflected in `aws_s3_meta_request_result.error_code`.
+ * If you're not sure which error to raise, use AWS_ERROR_S3_CANCELED.
+ *
+ * WARNING: This feature is experimental/unstable.
+ * At this time, the callback is only invoked for multipart upload
+ * (when Content-Length is above the `multipart_upload_threshold`,
+ * or Content-Length not specified).
+ */
+typedef int(aws_s3_meta_request_upload_review_fn)(
+    struct aws_s3_meta_request *meta_request,
+    const struct aws_s3_upload_review *review,
+    void *user_data);
+
+/**
+ * The factory function for S3 client to create a S3 Express credentials provider.
+ * The S3 client will be the only owner of the S3 Express credentials provider.
+ *
+ * During S3 client destruction, S3 client will start the destruction of the provider, and wait the
+ * on_provider_shutdown_callback to be invoked before the S3 client finish destruction.
+ *
+ * Note to implement the factory properly:
+ * - Make sure `on_provider_shutdown_callback` will be invoked after the provider finish shutdown, otherwise,
+ * leak will happen.
+ * - The provider must not acquire a reference to the client; otherwise, a circular reference will cause a deadlock.
+ * - The `client` provided CANNOT be used within the factory function call or the destructor.
+ *
+ * @param allocator    memory allocator to create the provider.
+ * @param client    The S3 client uses and owns the provider.
+ * @param on_provider_shutdown_callback    The callback to be invoked when the provider finishes shutdown.
+ * @param shutdown_user_data    The user data to invoke shutdown callback with
+ * @param user_data    The user data with the factory
+ *
+ * @return The aws_s3express_credentials_provider.
+ */
+typedef struct aws_s3express_credentials_provider *(aws_s3express_provider_factory_fn)(
+    struct aws_allocator *allocator,
+    struct aws_s3_client *client,
+    aws_simple_completion_callback on_provider_shutdown_callback,
+    void *shutdown_user_data,
+    void *factory_user_data);
+
 /* Keepalive properties are TCP only.
  * If interval or timeout are zero, then default values are used.
  */
@@ -239,7 +353,7 @@ struct aws_s3_client_config {
      * throughput_target_gbps. (Recommended) */
     uint32_t max_active_connections_override;
 
-    /* Region that the S3 bucket lives in. */
+    /* Region that the client default to. */
     struct aws_byte_cursor region;
 
     /* Client bootstrap used for common staples such as event loop group, host resolver, etc.. s*/
@@ -258,7 +372,21 @@ struct aws_s3_client_config {
      * is ENABLED, this is required. Otherwise, this is optional. */
     struct aws_tls_connection_options *tls_connection_options;
 
-    /* Signing options to be used for each request. Specify NULL to not sign requests. */
+    /**
+     * Required.
+     * Configure the signing for the requests made from the client.
+     * - Credentials or credentials provider is required. Other configs are all optional, and will be default to what
+     *      needs to sign the request for S3, only overrides when Non-zero/Not-empty is set.
+     * - To skip signing, you can config it with anonymous credentials.
+     * - S3 Client will derive the right config for signing process based on this.
+     *
+     * Notes:
+     * - For AWS_SIGNING_ALGORITHM_V4_S3EXPRESS, S3 client will use the credentials in the config to derive the
+     * S3 Express credentials that are used in the signing process.
+     * - For other auth algorithm, client may make modifications to signing config before passing it on to signer.
+     *
+     * TODO: deprecate this structure from auth, introduce a new S3 specific one.
+     */
     struct aws_signing_config_aws *signing_config;
 
     /* Size of parts the files will be downloaded or uploaded in. */
@@ -278,6 +406,9 @@ struct aws_s3_client_config {
     /* Throughput target in Gbps that we are trying to reach. */
     double throughput_target_gbps;
 
+    /* How much memory can we use. */
+    size_t memory_limit_in_bytes;
+
     /* Retry strategy to use. If NULL, a default retry strategy will be used. */
     struct aws_retry_strategy *retry_strategy;
 
@@ -351,6 +482,22 @@ struct aws_s3_client_config {
      * Ignored unless `enable_read_backpressure` is true.
      */
     size_t initial_read_window;
+
+    /**
+     * To enable S3 Express support or not.
+     */
+    bool enable_s3express;
+
+    /**
+     * Optional.
+     * Only used when `enable_s3express` is set.
+     *
+     * If set, client will invoke the factory to get the provider to use, when needed.
+     *
+     * If not set, client will create a default S3 Express provider under the hood.
+     */
+    aws_s3express_provider_factory_fn *s3express_provider_override_factory;
+    void *factory_user_data;
 };
 
 struct aws_s3_checksum_config {
@@ -375,7 +522,7 @@ struct aws_s3_checksum_config {
     enum aws_s3_checksum_algorithm checksum_algorithm;
 
     /**
-     * Enable checksum mode header will be attached to get requests, this will tell s3 to send back checksums headers if
+     * Enable checksum mode header will be attached to GET requests, this will tell s3 to send back checksums headers if
      * they exist. Calculate the corresponding checksum on the response bodies. The meta request will finish with a did
      * validate field and set the error code to AWS_ERROR_S3_RESPONSE_CHECKSUM_MISMATCH if the calculated
      * checksum, and checksum found in the response header do not match.
@@ -412,8 +559,31 @@ struct aws_s3_meta_request_options {
     /* The type of meta request we will be trying to accelerate. */
     enum aws_s3_meta_request_type type;
 
-    /* Signing options to be used for each request created for this meta request.  If NULL, options in the client will
-     * be used. If not NULL, these options will override the client options. */
+    /**
+     * Optional.
+     * The S3 operation name (e.g. "CreateBucket").
+     * This will only be used when type is AWS_S3_META_REQUEST_TYPE_DEFAULT;
+     * it is automatically populated for other meta-request types.
+     * This name is used to fill out details in metrics and error reports.
+     */
+    struct aws_byte_cursor operation_name;
+
+    /**
+     * Configure the signing for each request created for this meta request. If NULL, options in the client will be
+     *  used.
+     * - The credentials will be obtained based on the precedence of:
+     *      1. `credentials` from `signing_config` in `aws_s3_meta_request_options`
+     *      2. `credentials_provider` from `signing_config` in `aws_s3_meta_request_options`
+     *      3. `credentials` from `signing_config` cached in the client
+     *      4. `credentials_provider` cached in the client
+     * - To skip signing, you can config it with anonymous credentials.
+     * - S3 Client will derive the right config for signing process based on this.
+     *
+     * Notes:
+     * - For AWS_SIGNING_ALGORITHM_V4_S3EXPRESS, S3 client will use the credentials in the config to derive the
+     * S3 Express credentials that are used in the signing process.
+     * - For other auth algorithm, client may make modifications to signing config before passing it on to signer.
+     **/
     const struct aws_signing_config_aws *signing_config;
 
     /* Initial HTTP message that defines what operation we are doing.
@@ -472,6 +642,7 @@ struct aws_s3_meta_request_options {
 
     /**
      * Invoked to report progress of the meta request execution.
+     * See `aws_s3_meta_request_progress_fn`.
      */
     aws_s3_meta_request_progress_fn *progress_callback;
 
@@ -487,6 +658,14 @@ struct aws_s3_meta_request_options {
      */
     aws_s3_meta_request_telemetry_fn *telemetry_callback;
 
+    /**
+     * Optional.
+     * Callback for reviewing an upload before it completes.
+     * WARNING: experimental/unstable
+     * See `aws_s3_upload_review_fn`
+     */
+    aws_s3_meta_request_upload_review_fn *upload_review_callback;
+
     /**
      * Optional.
      * Endpoint override for request. Can be used to override scheme and port of
@@ -524,12 +703,23 @@ struct aws_s3_meta_request_options {
  */
 struct aws_s3_meta_request_result {
 
-    /* HTTP Headers for the failed request that triggered finish of the meta request.  NULL if no request failed. */
+    /* If meta request failed due to an HTTP error response from S3, these are the headers.
+     * NULL if meta request failed for another reason. */
     struct aws_http_headers *error_response_headers;
 
-    /* Response body for the failed request that triggered finishing of the meta request.  NUll if no request failed.*/
+    /* If meta request failed due to an HTTP error response from S3, this the body.
+     * NULL if meta request failed for another reason, or if the response had no body (such as a HEAD response). */
     struct aws_byte_buf *error_response_body;
 
+    /* If meta request failed due to an HTTP error response from S3,
+     * this is the name of the S3 operation it was responding to.
+     * For example, if a AWS_S3_META_REQUEST_TYPE_PUT_OBJECT fails this could be
+     * "PutObject, "CreateMultipartUpload", "UploadPart", "CompleteMultipartUpload", or others.
+     * For AWS_S3_META_REQUEST_TYPE_DEFAULT, this is the same value passed to
+     * aws_s3_meta_request_options.operation_name.
+     * NULL if the meta request failed for another reason, or the operation name is not known. */
+    struct aws_string *error_response_operation_name;
+
     /* Response status of the failed request or of the entire meta request. */
     int response_status;
 
@@ -745,6 +935,17 @@ void aws_s3_init_default_signing_config(
     const struct aws_byte_cursor region,
     struct aws_credentials_provider *credentials_provider);
 
+/**
+ * Return operation name for aws_s3_request_type,
+ * or empty string if the type doesn't map to an actual operation.
+ * For example:
+ * AWS_S3_REQUEST_TYPE_HEAD_OBJECT -> "HeadObject"
+ * AWS_S3_REQUEST_TYPE_UNKNOWN -> ""
+ * AWS_S3_REQUEST_TYPE_MAX -> ""
+ */
+AWS_S3_API
+const char *aws_s3_request_type_operation_name(enum aws_s3_request_type type);
+
 /**
  * Add a reference, keeping this object alive.
  * The reference must be released when you are done with it, or it's memory will never be cleaned up.
@@ -896,7 +1097,19 @@ int aws_s3_request_metrics_get_thread_id(const struct aws_s3_request_metrics *me
 AWS_S3_API
 int aws_s3_request_metrics_get_request_stream_id(const struct aws_s3_request_metrics *metrics, uint32_t *out_stream_id);
 
-/* Get the request type from request metrics. */
+/**
+ * Get the S3 operation name of the request (e.g. "HeadObject").
+ * If unavailable, AWS_ERROR_S3_METRIC_DATA_NOT_AVAILABLE will be raised.
+ * If available, out_operation_name will be set to a string.
+ * Be warned this string's lifetime is tied to the metrics object.
+ */
+AWS_S3_API
+int aws_s3_request_metrics_get_operation_name(
+    const struct aws_s3_request_metrics *metrics,
+    const struct aws_string **out_operation_name);
+
+/* Get the request type from request metrics.
+ * If you just need a string, aws_s3_request_metrics_get_operation_name() is more reliable. */
 AWS_S3_API
 void aws_s3_request_metrics_get_request_type(
     const struct aws_s3_request_metrics *metrics,
