@@ -5,6 +5,7 @@
 
 #include "aws/s3/private/s3_auto_ranged_get.h"
 #include "aws/s3/private/s3_auto_ranged_put.h"
+#include "aws/s3/private/s3_buffer_pool.h"
 #include "aws/s3/private/s3_client_impl.h"
 #include "aws/s3/private/s3_copy_object.h"
 #include "aws/s3/private/s3_default_meta_request.h"
@@ -12,6 +13,8 @@
 #include "aws/s3/private/s3_parallel_input_stream.h"
 #include "aws/s3/private/s3_request_messages.h"
 #include "aws/s3/private/s3_util.h"
+#include "aws/s3/private/s3express_credentials_provider_impl.h"
+#include "aws/s3/s3express_credentials_provider.h"
 
 #include <aws/auth/credentials.h>
 #include <aws/common/assert.h>
@@ -80,6 +83,9 @@ static size_t s_dns_host_address_ttl_seconds = 5 * 60;
  * 30 seconds mirrors the value currently used by the Java SDK. */
 static const uint32_t s_default_throughput_failure_interval_seconds = 30;
 
+/* Amount of time spent idling before trimming buffer. */
+static const size_t s_buffer_pool_trim_time_offset_in_s = 5;
+
 /* Called when ref count is 0. */
 static void s_s3_client_start_destroy(void *user_data);
 
@@ -219,6 +225,39 @@ void aws_s3_client_unlock_synced_data(struct aws_s3_client *client) {
     aws_mutex_unlock(&client->synced_data.lock);
 }
 
+static void s_s3express_provider_finish_destroy(void *user_data) {
+    struct aws_s3_client *client = user_data;
+    AWS_PRECONDITION(client);
+    /* BEGIN CRITICAL SECTION */
+    {
+        aws_s3_client_lock_synced_data(client);
+        client->synced_data.s3express_provider_active = false;
+        /* Schedule the work task to call s_s3_client_finish_destroy function if
+         * everything cleaning up asynchronously has finished. */
+        s_s3_client_schedule_process_work_synced(client);
+        aws_s3_client_unlock_synced_data(client);
+    }
+    /* END CRITICAL SECTION */
+}
+
+struct aws_s3express_credentials_provider *s_s3express_provider_default_factory(
+    struct aws_allocator *allocator,
+    struct aws_s3_client *client,
+    aws_simple_completion_callback on_provider_shutdown_callback,
+    void *shutdown_user_data,
+    void *factory_user_data) {
+    (void)factory_user_data;
+
+    struct aws_s3express_credentials_provider_default_options options = {
+        .client = client,
+        .shutdown_complete_callback = on_provider_shutdown_callback,
+        .shutdown_user_data = shutdown_user_data,
+    };
+    struct aws_s3express_credentials_provider *s3express_provider =
+        aws_s3express_credentials_provider_new_default(allocator, &options);
+    return s3express_provider;
+}
+
 struct aws_s3_client *aws_s3_client_new(
     struct aws_allocator *allocator,
     const struct aws_s3_client_config *client_config) {
@@ -243,6 +282,33 @@ struct aws_s3_client *aws_s3_client_new(
         return NULL;
     }
 
+    if (client_config->signing_config == NULL) {
+        AWS_LOGF_ERROR(AWS_LS_S3_CLIENT, "Cannot create client from client_config; signing_config is required.");
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        return NULL;
+    }
+
+    if (client_config->signing_config->credentials == NULL &&
+        client_config->signing_config->credentials_provider == NULL) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_CLIENT,
+            "Cannot create client from client_config; Invalid signing_config provided, either credentials or "
+            "credentials provider has to be set.");
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        return NULL;
+    }
+
+    if (!client_config->enable_s3express &&
+        client_config->signing_config->algorithm == AWS_SIGNING_ALGORITHM_V4_S3EXPRESS) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_CLIENT,
+            "Cannot create client from client_config; Client config is set use S3 Express signing, but S3 Express "
+            "support is "
+            "not configured.");
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        return NULL;
+    }
+
 #ifdef BYO_CRYPTO
     if (client_config->tls_mode == AWS_MR_TLS_ENABLED && client_config->tls_connection_options == NULL) {
         AWS_LOGF_ERROR(
@@ -257,12 +323,58 @@ struct aws_s3_client *aws_s3_client_new(
     struct aws_s3_client *client = aws_mem_calloc(allocator, 1, sizeof(struct aws_s3_client));
 
     client->allocator = allocator;
+
+    size_t mem_limit = 0;
+    if (client_config->memory_limit_in_bytes == 0) {
+#if SIZE_BITS == 32
+        if (client_config->throughput_target_gbps > 25.0) {
+            mem_limit = GB_TO_BYTES(2);
+        } else {
+            mem_limit = GB_TO_BYTES(1);
+        }
+#else
+        if (client_config->throughput_target_gbps > 75.0) {
+            mem_limit = GB_TO_BYTES(8);
+        } else if (client_config->throughput_target_gbps > 25.0) {
+            mem_limit = GB_TO_BYTES(4);
+        } else {
+            mem_limit = GB_TO_BYTES(2);
+        }
+#endif
+    } else {
+        mem_limit = client_config->memory_limit_in_bytes;
+    }
+
+    size_t part_size;
+    if (client_config->part_size != 0) {
+        part_size = (size_t)client_config->part_size;
+    } else {
+        part_size = s_default_part_size;
+    }
+
+    client->buffer_pool = aws_s3_buffer_pool_new(allocator, part_size, mem_limit);
+
+    if (client->buffer_pool == NULL) {
+        goto on_early_fail;
+    }
+
+    struct aws_s3_buffer_pool_usage_stats pool_usage = aws_s3_buffer_pool_get_usage(client->buffer_pool);
+
+    if (client_config->max_part_size > pool_usage.mem_limit) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_CLIENT,
+            "Cannot create client from client_config; configured max part size should not exceed memory limit."
+            "size.");
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        goto on_early_fail;
+    }
+
     client->vtable = &s_s3_client_default_vtable;
 
     aws_ref_count_init(&client->ref_count, client, (aws_simple_completion_callback *)s_s3_client_start_destroy);
 
     if (aws_mutex_init(&client->synced_data.lock) != AWS_OP_SUCCESS) {
-        goto lock_init_fail;
+        goto on_early_fail;
     }
 
     aws_linked_list_init(&client->synced_data.pending_meta_request_work);
@@ -293,17 +405,18 @@ struct aws_s3_client *aws_s3_client_new(
     /* Make a copy of the region string. */
     client->region = aws_string_new_from_array(allocator, client_config->region.ptr, client_config->region.len);
 
-    if (client_config->part_size != 0) {
-        *((size_t *)&client->part_size) = (size_t)client_config->part_size;
-    } else {
-        *((size_t *)&client->part_size) = s_default_part_size;
-    }
+    *((size_t *)&client->part_size) = part_size;
 
     if (client_config->max_part_size != 0) {
         *((uint64_t *)&client->max_part_size) = client_config->max_part_size;
     } else {
         *((uint64_t *)&client->max_part_size) = s_default_max_part_size;
     }
+
+    if (client_config->max_part_size > pool_usage.mem_limit) {
+        *((uint64_t *)&client->max_part_size) = pool_usage.mem_limit;
+    }
+
     if (client->max_part_size > SIZE_MAX) {
         /* For the 32bit max part size to be SIZE_MAX */
         *((uint64_t *)&client->max_part_size) = SIZE_MAX;
@@ -426,8 +539,14 @@ struct aws_s3_client *aws_s3_client_new(
         *((uint32_t *)&client->ideal_vip_count) = (uint32_t)ceil(ideal_vip_count_double);
     }
 
-    if (client_config->signing_config) {
-        client->cached_signing_config = aws_cached_signing_config_new(client->allocator, client_config->signing_config);
+    client->cached_signing_config = aws_cached_signing_config_new(client, client_config->signing_config);
+    if (client_config->enable_s3express) {
+        if (client_config->s3express_provider_override_factory) {
+            client->s3express_provider_factory = client_config->s3express_provider_override_factory;
+            client->factory_user_data = client_config->factory_user_data;
+        } else {
+            client->s3express_provider_factory = s_s3express_provider_default_factory;
+        }
     }
 
     client->synced_data.active = true;
@@ -488,7 +607,7 @@ on_error:
     aws_event_loop_group_release(client->client_bootstrap->event_loop_group);
     aws_client_bootstrap_release(client->client_bootstrap);
     aws_mutex_clean_up(&client->synced_data.lock);
-lock_init_fail:
+on_early_fail:
     aws_mem_release(client->allocator, client);
     return NULL;
 }
@@ -532,6 +651,7 @@ static void s_s3_client_start_destroy(void *user_data) {
 
     aws_event_loop_group_release(client->body_streaming_elg);
     client->body_streaming_elg = NULL;
+    aws_s3express_credentials_provider_release(client->s3express_provider);
 
     /* BEGIN CRITICAL SECTION */
     {
@@ -551,6 +671,10 @@ static void s_s3_client_finish_destroy_default(struct aws_s3_client *client) {
 
     AWS_LOGF_DEBUG(AWS_LS_S3_CLIENT, "id=%p Client finishing destruction.", (void *)client);
 
+    if (client->threaded_data.trim_buffer_pool_task_scheduled) {
+        aws_event_loop_cancel_task(client->process_work_event_loop, &client->synced_data.trim_buffer_pool_task);
+    }
+
     aws_string_destroy(client->region);
     client->region = NULL;
 
@@ -588,6 +712,7 @@ static void s_s3_client_finish_destroy_default(struct aws_s3_client *client) {
     aws_s3_client_shutdown_complete_callback_fn *shutdown_callback = client->shutdown_callback;
     void *shutdown_user_data = client->shutdown_callback_user_data;
 
+    aws_s3_buffer_pool_destroy(client->buffer_pool);
     aws_mem_release(client->allocator, client);
     client = NULL;
 
@@ -715,6 +840,13 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
     AWS_PRECONDITION(client->vtable->meta_request_factory);
     AWS_PRECONDITION(options);
 
+    bool use_s3express_signing = false;
+    if (options->signing_config != NULL) {
+        use_s3express_signing = options->signing_config->algorithm == AWS_SIGNING_ALGORITHM_V4_S3EXPRESS;
+    } else if (client->cached_signing_config) {
+        use_s3express_signing = client->cached_signing_config->config.algorithm == AWS_SIGNING_ALGORITHM_V4_S3EXPRESS;
+    }
+
     if (options->type >= AWS_S3_META_REQUEST_TYPE_MAX) {
         AWS_LOGF_ERROR(
             AWS_LS_S3_CLIENT,
@@ -733,6 +865,15 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
         return NULL;
     }
 
+    if (use_s3express_signing && client->s3express_provider_factory == NULL) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_CLIENT,
+            "id=%p Cannot create meta s3 request; client doesn't support S3 Express signing.",
+            (void *)client);
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        return NULL;
+    }
+
     struct aws_http_headers *message_headers = aws_http_message_get_headers(options->message);
 
     if (message_headers == NULL) {
@@ -834,6 +975,34 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
     {
         aws_s3_client_lock_synced_data(client);
 
+        if (use_s3express_signing && !client->synced_data.s3express_provider_active) {
+
+            AWS_LOGF_TRACE(AWS_LS_S3_CLIENT, "id=%p Create S3 Express provider for the client.", (void *)client);
+            /**
+             * Invoke the factory within the lock. We WARNED people uses their own factory to not use ANY client related
+             * api during the factory.
+             *
+             * We cannot just release the lock and invoke the factory, because it can lead to the other request assume
+             * the provider is active, and not waiting for the provider to be created. And lead to unexpected behavior.
+             */
+            client->s3express_provider = client->s3express_provider_factory(
+                client->allocator, client, s_s3express_provider_finish_destroy, client, client->factory_user_data);
+
+            /* Provider is related to client, we don't need to clean it up if meta request failed. But, if provider
+             * failed to be created, let's bail out earlier. */
+            if (!client->s3express_provider) {
+                AWS_LOGF_ERROR(
+                    AWS_LS_S3_CLIENT,
+                    "id=%p Failed to create S3 Express provider for client due to error %d (%s)",
+                    (void *)client,
+                    aws_last_error_or_unknown(),
+                    aws_error_str(aws_last_error_or_unknown()));
+                error_occurred = true;
+                goto unlock;
+            }
+            client->synced_data.s3express_provider_active = true;
+        }
+
         struct aws_string *endpoint_host_name = NULL;
 
         if (options->endpoint != NULL) {
@@ -852,6 +1021,10 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
         struct aws_s3_endpoint *endpoint = NULL;
         struct aws_hash_element *endpoint_hash_element = NULL;
 
+        if (use_s3express_signing) {
+            meta_request->s3express_session_host = aws_string_new_from_string(client->allocator, endpoint_host_name);
+        }
+
         int was_created = 0;
         if (aws_hash_table_create(
                 &client->synced_data.endpoints, endpoint_host_name, &endpoint_hash_element, &was_created)) {
@@ -987,7 +1160,13 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
              * TODO: Still need tests to verify that the request of a part is
              * splittable or not */
             if (aws_http_headers_has(initial_message_headers, aws_byte_cursor_from_c_str("partNumber"))) {
-                return aws_s3_meta_request_default_new(client->allocator, client, content_length, false, options);
+                return aws_s3_meta_request_default_new(
+                    client->allocator,
+                    client,
+                    AWS_S3_REQUEST_TYPE_GET_OBJECT,
+                    content_length,
+                    false /*should_compute_content_md5*/,
+                    options);
             }
 
             return aws_s3_meta_request_auto_ranged_get_new(client->allocator, client, client->part_size, options);
@@ -1036,6 +1215,7 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
                     return aws_s3_meta_request_default_new(
                         client->allocator,
                         client,
+                        AWS_S3_REQUEST_TYPE_PUT_OBJECT,
                         content_length,
                         client->compute_content_md5 == AWS_MR_CONTENT_MD5_ENABLED &&
                             !aws_http_headers_has(initial_message_headers, g_content_md5_header_name),
@@ -1082,7 +1262,13 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
             return aws_s3_meta_request_copy_object_new(client->allocator, client, options);
         }
         case AWS_S3_META_REQUEST_TYPE_DEFAULT:
-            return aws_s3_meta_request_default_new(client->allocator, client, content_length, false, options);
+            return aws_s3_meta_request_default_new(
+                client->allocator,
+                client,
+                AWS_S3_REQUEST_TYPE_UNKNOWN,
+                content_length,
+                false /*should_compute_content_md5*/,
+                options);
         default:
             AWS_FATAL_ASSERT(false);
     }
@@ -1100,8 +1286,7 @@ static void s_s3_client_push_meta_request_synced(
     struct aws_s3_meta_request_work *meta_request_work =
         aws_mem_calloc(client->allocator, 1, sizeof(struct aws_s3_meta_request_work));
 
-    aws_s3_meta_request_acquire(meta_request);
-    meta_request_work->meta_request = meta_request;
+    meta_request_work->meta_request = aws_s3_meta_request_acquire(meta_request);
     aws_linked_list_push_back(&client->synced_data.pending_meta_request_work, &meta_request_work->node);
 }
 
@@ -1130,6 +1315,57 @@ static void s_s3_client_schedule_process_work_synced_default(struct aws_s3_clien
     client->synced_data.process_work_task_scheduled = true;
 }
 
+/* Task function for trying to find a request that can be processed. */
+static void s_s3_client_trim_buffer_pool_task(struct aws_task *task, void *arg, enum aws_task_status task_status) {
+    AWS_PRECONDITION(task);
+    (void)task;
+    (void)task_status;
+
+    if (task_status != AWS_TASK_STATUS_RUN_READY) {
+        return;
+    }
+
+    struct aws_s3_client *client = arg;
+    AWS_PRECONDITION(client);
+
+    client->threaded_data.trim_buffer_pool_task_scheduled = false;
+
+    uint32_t num_reqs_in_flight = (uint32_t)aws_atomic_load_int(&client->stats.num_requests_in_flight);
+
+    if (num_reqs_in_flight == 0) {
+        aws_s3_buffer_pool_trim(client->buffer_pool);
+    }
+}
+
+static void s_s3_client_schedule_buffer_pool_trim_synced(struct aws_s3_client *client) {
+    ASSERT_SYNCED_DATA_LOCK_HELD(client);
+
+    if (client->threaded_data.trim_buffer_pool_task_scheduled) {
+        return;
+    }
+
+    uint32_t num_reqs_in_flight = (uint32_t)aws_atomic_load_int(&client->stats.num_requests_in_flight);
+    if (num_reqs_in_flight > 0) {
+        return;
+    }
+
+    aws_task_init(
+        &client->synced_data.trim_buffer_pool_task,
+        s_s3_client_trim_buffer_pool_task,
+        client,
+        "s3_client_buffer_pool_trim_task");
+
+    uint64_t trim_time = 0;
+    aws_event_loop_current_clock_time(client->process_work_event_loop, &trim_time);
+    trim_time +=
+        aws_timestamp_convert(s_buffer_pool_trim_time_offset_in_s, AWS_TIMESTAMP_SECS, AWS_TIMESTAMP_NANOS, NULL);
+
+    aws_event_loop_schedule_task_future(
+        client->process_work_event_loop, &client->synced_data.trim_buffer_pool_task, trim_time);
+
+    client->threaded_data.trim_buffer_pool_task_scheduled = true;
+}
+
 void aws_s3_client_schedule_process_work(struct aws_s3_client *client) {
     AWS_PRECONDITION(client);
 
@@ -1193,6 +1429,10 @@ static void s_s3_client_process_work_default(struct aws_s3_client *client) {
     client->synced_data.process_work_task_scheduled = false;
     client->synced_data.process_work_task_in_progress = true;
 
+    if (client->synced_data.active) {
+        s_s3_client_schedule_buffer_pool_trim_synced(client);
+    }
+
     aws_linked_list_swap_contents(&meta_request_work_list, &client->synced_data.pending_meta_request_work);
 
     uint32_t num_requests_queued =
@@ -1326,12 +1566,12 @@ static void s_s3_client_process_work_default(struct aws_s3_client *client) {
         /* This flag should never be set twice. If it was, that means a double-free could occur.*/
         AWS_ASSERT(!client->synced_data.finish_destroy);
 
-        bool finish_destroy = client->synced_data.active == false &&
-                              client->synced_data.start_destroy_executing == false &&
-                              client->synced_data.body_streaming_elg_allocated == false &&
-                              client->synced_data.process_work_task_scheduled == false &&
-                              client->synced_data.process_work_task_in_progress == false &&
-                              client->synced_data.num_endpoints_allocated == 0;
+        bool finish_destroy =
+            client->synced_data.active == false && client->synced_data.start_destroy_executing == false &&
+            client->synced_data.body_streaming_elg_allocated == false &&
+            client->synced_data.process_work_task_scheduled == false &&
+            client->synced_data.process_work_task_in_progress == false &&
+            client->synced_data.s3express_provider_active == false && client->synced_data.num_endpoints_allocated == 0;
 
         client->synced_data.finish_destroy = finish_destroy;
 
@@ -1340,13 +1580,14 @@ static void s_s3_client_process_work_default(struct aws_s3_client *client) {
                 AWS_LS_S3_CLIENT,
                 "id=%p Client shutdown progress: starting_destroy_executing=%d  body_streaming_elg_allocated=%d  "
                 "process_work_task_scheduled=%d  process_work_task_in_progress=%d  num_endpoints_allocated=%d "
-                "finish_destroy=%d",
+                "s3express_provider_active=%d finish_destroy=%d",
                 (void *)client,
                 (int)client->synced_data.start_destroy_executing,
                 (int)client->synced_data.body_streaming_elg_allocated,
                 (int)client->synced_data.process_work_task_scheduled,
                 (int)client->synced_data.process_work_task_in_progress,
                 (int)client->synced_data.num_endpoints_allocated,
+                (int)client->synced_data.s3express_provider_active,
                 (int)client->synced_data.finish_destroy);
         }
 
@@ -1383,6 +1624,8 @@ void aws_s3_client_update_meta_requests_threaded(struct aws_s3_client *client) {
 
     const uint32_t num_passes = AWS_ARRAY_SIZE(pass_flags);
 
+    aws_s3_buffer_pool_remove_reservation_hold(client->buffer_pool);
+
     for (uint32_t pass_index = 0; pass_index < num_passes; ++pass_index) {
 
         /* While:
@@ -1425,6 +1668,9 @@ void aws_s3_client_update_meta_requests_threaded(struct aws_s3_client *client) {
             struct aws_s3_request *request = NULL;
 
             /* Try to grab the next request from the meta request. */
+            /* TODO: should we bail out if request fails to update due to mem or
+             * continue going and hopping that following reqs can fit into mem?
+             * check if avail space is at least part size? */
             bool work_remaining = aws_s3_meta_request_update(meta_request, pass_flags[pass_index], &request);
 
             if (work_remaining) {
@@ -1860,7 +2106,6 @@ reset_connection:
     }
 
     if (connection->request != NULL) {
-
         connection->request = aws_s3_request_release(connection->request);
     }
 
@@ -2057,7 +2302,7 @@ struct aws_byte_cursor aws_s3_meta_request_resume_token_upload_id(
 
 static uint64_t s_upload_timeout_threshold_ns = 5000000000; /* 5 Secs */
 const size_t g_expect_timeout_offset_ms =
-    700; /* 0.7 Secs. From experienments on c5n.18xlarge machine for 30 GiB upload, it gave us best performance. */
+    700; /* 0.7 Secs. From experiments on c5n.18xlarge machine for 30 GiB upload, it gave us best performance. */
 
 /**
  * The upload timeout optimization: explained.
