@@ -10,6 +10,7 @@
 #include "aws/s3/private/s3_parallel_input_stream.h"
 #include "aws/s3/private/s3_request_messages.h"
 #include "aws/s3/private/s3_util.h"
+#include "aws/s3/s3express_credentials_provider.h"
 #include <aws/auth/signable.h>
 #include <aws/auth/signing.h>
 #include <aws/auth/signing_config.h>
@@ -102,8 +103,8 @@ static int s_meta_request_get_response_headers_checksum_callback(
             if (header_sum.len == encoded_len - 1) {
                 /* encoded_len includes the nullptr length. -1 is the expected length. */
                 aws_byte_buf_init_copy_from_cursor(
-                    &meta_request->meta_request_level_response_header_checksum, aws_default_allocator(), header_sum);
-                meta_request->meta_request_level_running_response_sum = aws_checksum_new(aws_default_allocator(), i);
+                    &meta_request->meta_request_level_response_header_checksum, meta_request->allocator, header_sum);
+                meta_request->meta_request_level_running_response_sum = aws_checksum_new(meta_request->allocator, i);
             }
             break;
         }
@@ -151,10 +152,10 @@ static void s_meta_request_get_response_finish_checksum_callback(
         /* what error should I raise for these? */
         aws_base64_compute_encoded_len(
             meta_request->meta_request_level_running_response_sum->digest_size, &encoded_checksum_len);
-        aws_byte_buf_init(&encoded_response_body_sum, aws_default_allocator(), encoded_checksum_len);
+        aws_byte_buf_init(&encoded_response_body_sum, meta_request->allocator, encoded_checksum_len);
         aws_byte_buf_init(
             &response_body_sum,
-            aws_default_allocator(),
+            meta_request->allocator,
             meta_request->meta_request_level_running_response_sum->digest_size);
         aws_checksum_finalize(meta_request->meta_request_level_running_response_sum, &response_body_sum, 0);
         struct aws_byte_cursor response_body_sum_cursor = aws_byte_cursor_from_buf(&response_body_sum);
@@ -241,8 +242,9 @@ int aws_s3_meta_request_init_base(
     *((size_t *)&meta_request->part_size) = part_size;
     *((bool *)&meta_request->should_compute_content_md5) = should_compute_content_md5;
     checksum_config_init(&meta_request->checksum_config, options->checksum_config);
+
     if (options->signing_config) {
-        meta_request->cached_signing_config = aws_cached_signing_config_new(allocator, options->signing_config);
+        meta_request->cached_signing_config = aws_cached_signing_config_new(client, options->signing_config);
     }
 
     /* Client is currently optional to allow spinning up a meta_request without a client in a test. */
@@ -456,6 +458,7 @@ struct aws_s3_meta_request *aws_s3_meta_request_release(struct aws_s3_meta_reque
 static void s_s3_meta_request_destroy(void *user_data) {
     struct aws_s3_meta_request *meta_request = user_data;
     AWS_PRECONDITION(meta_request);
+    void *log_id = meta_request;
 
     AWS_LOGF_DEBUG(AWS_LS_S3_META_REQUEST, "id=%p Cleaning up meta request", (void *)meta_request);
 
@@ -467,6 +470,7 @@ static void s_s3_meta_request_destroy(void *user_data) {
     aws_s3_meta_request_shutdown_fn *shutdown_callback = meta_request->shutdown_callback;
 
     aws_cached_signing_config_destroy(meta_request->cached_signing_config);
+    aws_string_destroy(meta_request->s3express_session_host);
     aws_mutex_clean_up(&meta_request->synced_data.lock);
     /* endpoint should have already been released and set NULL by the meta request finish call.
      * But call release() again, just in case we're tearing down a half-initialized meta request */
@@ -485,18 +489,17 @@ static void s_s3_meta_request_destroy(void *user_data) {
     aws_s3_meta_request_result_clean_up(meta_request, &meta_request->synced_data.finish_result);
 
     if (meta_request->vtable != NULL) {
-        AWS_LOGF_TRACE(
-            AWS_LS_S3_META_REQUEST, "id=%p Calling virtual meta request destroy function.", (void *)meta_request);
+        AWS_LOGF_TRACE(AWS_LS_S3_META_REQUEST, "id=%p Calling virtual meta request destroy function.", log_id);
         meta_request->vtable->destroy(meta_request);
     }
     meta_request = NULL;
 
     if (shutdown_callback != NULL) {
-        AWS_LOGF_TRACE(AWS_LS_S3_META_REQUEST, "id=%p Calling meta request shutdown callback.", (void *)meta_request);
+        AWS_LOGF_TRACE(AWS_LS_S3_META_REQUEST, "id=%p Calling meta request shutdown callback.", log_id);
         shutdown_callback(meta_request_user_data);
     }
 
-    AWS_LOGF_TRACE(AWS_LS_S3_META_REQUEST, "id=%p Meta request clean up finished.", (void *)meta_request);
+    AWS_LOGF_TRACE(AWS_LS_S3_META_REQUEST, "id=%p Meta request clean up finished.", log_id);
 }
 
 static int s_s3_request_priority_queue_pred(const void *a, const void *b) {
@@ -720,9 +723,147 @@ static void s_s3_meta_request_sign_request(
     AWS_PRECONDITION(meta_request->vtable);
     AWS_PRECONDITION(meta_request->vtable->sign_request);
 
+    if (request->send_data.metrics) {
+        struct aws_s3_request_metrics *metric = request->send_data.metrics;
+        aws_high_res_clock_get_ticks((uint64_t *)&metric->time_metrics.sign_start_timestamp_ns);
+    }
+
     meta_request->vtable->sign_request(meta_request, request, on_signing_complete, user_data);
 }
 
+struct aws_get_s3express_credentials_user_data {
+    /* Keep our own reference to allocator, because the meta request can be gone after the callback invoked. */
+    struct aws_allocator *allocator;
+
+    struct aws_s3_meta_request *meta_request;
+    struct aws_s3_request *request;
+    aws_signing_complete_fn *on_signing_complete;
+
+    const struct aws_credentials *original_credentials;
+
+    struct aws_signing_config_aws base_signing_config;
+    struct aws_credentials_properties_s3express properties;
+    void *user_data;
+};
+
+static void s_aws_get_s3express_credentials_user_data_destroy(struct aws_get_s3express_credentials_user_data *context) {
+    aws_s3_meta_request_release(context->meta_request);
+    aws_credentials_release(context->original_credentials);
+    aws_mem_release(context->allocator, context);
+}
+
+static void s_get_s3express_credentials_callback(struct aws_credentials *credentials, int error_code, void *user_data) {
+    struct aws_get_s3express_credentials_user_data *context = user_data;
+    struct aws_signing_config_aws signing_config = context->base_signing_config;
+
+    if (error_code) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p: Failed to get S3 Express credentials %p. due to error code %d (%s)",
+            (void *)context->meta_request,
+            (void *)context->request,
+            error_code,
+            aws_error_str(error_code));
+        context->on_signing_complete(NULL, error_code, context->user_data);
+        goto done;
+    }
+    s_s3_meta_request_init_signing_date_time(context->meta_request, &signing_config.date);
+    /* Override the credentials */
+    signing_config.credentials = credentials;
+    signing_config.algorithm = AWS_SIGNING_ALGORITHM_V4_S3EXPRESS;
+    if (aws_sign_request_aws(
+            context->allocator,
+            context->request->send_data.signable,
+            (struct aws_signing_config_base *)&signing_config,
+            context->on_signing_complete,
+            context->user_data)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p: Could not sign request %p. due to error code %d (%s)",
+            (void *)context->meta_request,
+            (void *)context->request,
+            aws_last_error_or_unknown(),
+            aws_error_str(aws_last_error_or_unknown()));
+        context->on_signing_complete(NULL, aws_last_error_or_unknown(), context->user_data);
+    }
+done:
+    s_aws_get_s3express_credentials_user_data_destroy(context);
+}
+
+static void s_get_original_credentials_callback(struct aws_credentials *credentials, int error_code, void *user_data) {
+    struct aws_get_s3express_credentials_user_data *context = user_data;
+    struct aws_s3_meta_request *meta_request = context->meta_request;
+    if (error_code) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p: Failed to get S3 Express credentials %p. due to error code %d (%s)",
+            (void *)context->meta_request,
+            (void *)context->request,
+            error_code,
+            aws_error_str(error_code));
+        context->on_signing_complete(NULL, error_code, context->user_data);
+        s_aws_get_s3express_credentials_user_data_destroy(context);
+        return;
+    }
+    context->original_credentials = credentials;
+    aws_credentials_acquire(context->original_credentials);
+
+    /**
+     * Derive the credentials for S3 Express.
+     */
+    struct aws_s3_client *client = meta_request->client;
+    if (aws_s3express_credentials_provider_get_credentials(
+            client->s3express_provider,
+            context->original_credentials,
+            &context->properties,
+            s_get_s3express_credentials_callback,
+            context)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p: Could not get S3 Express credentials %p",
+            (void *)meta_request,
+            (void *)context->request);
+        context->on_signing_complete(NULL, aws_last_error_or_unknown(), user_data);
+        s_aws_get_s3express_credentials_user_data_destroy(context);
+    }
+}
+
+static int s_meta_request_resolve_signing_config(
+    struct aws_signing_config_aws *out_signing_config,
+    struct aws_s3_request *request,
+    struct aws_s3_meta_request *meta_request) {
+    struct aws_s3_client *client = meta_request->client;
+    if (meta_request->cached_signing_config != NULL) {
+        *out_signing_config = meta_request->cached_signing_config->config;
+
+        if (out_signing_config->credentials == NULL && out_signing_config->credentials_provider == NULL) {
+            /* When no credentials available from meta request level override, we use the credentials from client */
+            out_signing_config->credentials = client->cached_signing_config->config.credentials;
+            out_signing_config->credentials_provider = client->cached_signing_config->config.credentials_provider;
+        }
+    } else if (client->cached_signing_config != NULL) {
+        *out_signing_config = client->cached_signing_config->config;
+    } else {
+        /* Not possible to have no cached signing config from both client and request */
+        AWS_FATAL_ASSERT(false);
+    }
+
+    /* If the checksum is configured to be added to the trailer, the payload will be aws-chunked encoded. The payload
+     * will need to be streaming signed/unsigned. */
+    if (meta_request->checksum_config.location == AWS_SCL_TRAILER &&
+        aws_byte_cursor_eq(&out_signing_config->signed_body_value, &g_aws_signed_body_value_unsigned_payload)) {
+        out_signing_config->signed_body_value = g_aws_signed_body_value_streaming_unsigned_payload_trailer;
+    }
+    /* However the initial request for a multipart upload does not have a trailing checksum and is not chunked so it
+     * must have an unsigned_payload signed_body value*/
+    if (request->part_number == 0 &&
+        aws_byte_cursor_eq(
+            &out_signing_config->signed_body_value, &g_aws_signed_body_value_streaming_unsigned_payload_trailer)) {
+        out_signing_config->signed_body_value = g_aws_signed_body_value_unsigned_payload;
+    }
+    return AWS_OP_SUCCESS;
+}
+
 /* Handles signing a message for the caller. */
 void aws_s3_meta_request_sign_request_default(
     struct aws_s3_meta_request *meta_request,
@@ -738,11 +879,7 @@ void aws_s3_meta_request_sign_request_default(
 
     struct aws_signing_config_aws signing_config;
 
-    if (meta_request->cached_signing_config != NULL) {
-        signing_config = meta_request->cached_signing_config->config;
-    } else if (client->cached_signing_config != NULL) {
-        signing_config = client->cached_signing_config->config;
-    } else {
+    if (s_meta_request_resolve_signing_config(&signing_config, request, meta_request)) {
         AWS_LOGF_DEBUG(
             AWS_LS_S3_META_REQUEST,
             "id=%p: No signing config present. Not signing request %p.",
@@ -753,8 +890,6 @@ void aws_s3_meta_request_sign_request_default(
         return;
     }
 
-    s_s3_meta_request_init_signing_date_time(meta_request, &signing_config.date);
-
     request->send_data.signable = aws_signable_new_http_request(meta_request->allocator, request->send_data.message);
 
     AWS_LOGF_TRACE(
@@ -776,32 +911,71 @@ void aws_s3_meta_request_sign_request_default(
         return;
     }
 
-    /* If the checksum is configured to be added to the trailer, the payload will be aws-chunked encoded. The payload
-     * will need to be streaming signed/unsigned. */
-    if (meta_request->checksum_config.location == AWS_SCL_TRAILER &&
-        aws_byte_cursor_eq(&signing_config.signed_body_value, &g_aws_signed_body_value_unsigned_payload)) {
-        signing_config.signed_body_value = g_aws_signed_body_value_streaming_unsigned_payload_trailer;
-    }
-    /* However the initial request for a multipart upload does not have a trailing checksum and is not chunked so it
-     * must have an unsigned_payload signed_body value*/
-    if (request->part_number == 0 &&
-        aws_byte_cursor_eq(
-            &signing_config.signed_body_value, &g_aws_signed_body_value_streaming_unsigned_payload_trailer)) {
-        signing_config.signed_body_value = g_aws_signed_body_value_unsigned_payload;
-    }
+    if (signing_config.algorithm == AWS_SIGNING_ALGORITHM_V4_S3EXPRESS) {
+        /* Fetch credentials from S3 Express provider. */
+        struct aws_get_s3express_credentials_user_data *context =
+            aws_mem_calloc(meta_request->allocator, 1, sizeof(struct aws_get_s3express_credentials_user_data));
 
-    if (aws_sign_request_aws(
-            meta_request->allocator,
-            request->send_data.signable,
-            (struct aws_signing_config_base *)&signing_config,
-            on_signing_complete,
-            user_data)) {
+        context->allocator = meta_request->allocator;
+        context->base_signing_config = signing_config;
+        context->meta_request = aws_s3_meta_request_acquire(meta_request);
+        context->on_signing_complete = on_signing_complete;
+        context->request = request;
+        context->user_data = user_data;
+        context->properties.host = aws_byte_cursor_from_string(meta_request->s3express_session_host);
+        context->properties.region = signing_config.region;
 
-        AWS_LOGF_ERROR(
-            AWS_LS_S3_META_REQUEST, "id=%p: Could not sign request %p", (void *)meta_request, (void *)request);
+        if (signing_config.credentials) {
+            context->original_credentials = signing_config.credentials;
+            aws_credentials_acquire(context->original_credentials);
+            /**
+             * Derive the credentials for S3 Express.
+             */
+            if (aws_s3express_credentials_provider_get_credentials(
+                    client->s3express_provider,
+                    context->original_credentials,
+                    &context->properties,
+                    s_get_s3express_credentials_callback,
+                    context)) {
+                AWS_LOGF_ERROR(
+                    AWS_LS_S3_META_REQUEST,
+                    "id=%p: Could not get S3 Express credentials %p",
+                    (void *)meta_request,
+                    (void *)request);
+                on_signing_complete(NULL, aws_last_error_or_unknown(), user_data);
+                s_aws_get_s3express_credentials_user_data_destroy(context);
+                return;
+            }
+        } else if (signing_config.credentials_provider) {
+            /* Get the credentials from provider first. */
+            if (aws_credentials_provider_get_credentials(
+                    signing_config.credentials_provider, s_get_original_credentials_callback, context)) {
+                AWS_LOGF_ERROR(
+                    AWS_LS_S3_META_REQUEST,
+                    "id=%p: Could not get S3 Express credentials %p",
+                    (void *)meta_request,
+                    (void *)request);
+                on_signing_complete(NULL, aws_last_error_or_unknown(), user_data);
+                s_aws_get_s3express_credentials_user_data_destroy(context);
+                return;
+            }
+        }
+    } else {
+        /* Regular signing. */
+        s_s3_meta_request_init_signing_date_time(meta_request, &signing_config.date);
+        if (aws_sign_request_aws(
+                meta_request->allocator,
+                request->send_data.signable,
+                (struct aws_signing_config_base *)&signing_config,
+                on_signing_complete,
+                user_data)) {
 
-        on_signing_complete(NULL, aws_last_error_or_unknown(), user_data);
-        return;
+            AWS_LOGF_ERROR(
+                AWS_LS_S3_META_REQUEST, "id=%p: Could not sign request %p", (void *)meta_request, (void *)request);
+
+            on_signing_complete(NULL, aws_last_error_or_unknown(), user_data);
+            return;
+        }
     }
 }
 
@@ -832,6 +1006,14 @@ static void s_s3_meta_request_request_on_signed(
         goto finish;
     }
 
+    if (request->send_data.metrics) {
+        struct aws_s3_request_metrics *metric = request->send_data.metrics;
+        aws_high_res_clock_get_ticks((uint64_t *)&metric->time_metrics.sign_end_timestamp_ns);
+        AWS_ASSERT(metric->time_metrics.sign_start_timestamp_ns != 0);
+        metric->time_metrics.signing_duration_ns =
+            metric->time_metrics.sign_end_timestamp_ns - metric->time_metrics.sign_start_timestamp_ns;
+    }
+
 finish:
 
     if (error_code != AWS_ERROR_SUCCESS) {
@@ -847,12 +1029,6 @@ finish:
     s_s3_prepare_request_payload_callback_and_destroy(payload, error_code);
 }
 
-static bool s_s3_request_is_upload_part(struct aws_s3_request *request) {
-    struct aws_s3_meta_request *meta_request = request->meta_request;
-    return meta_request->type == AWS_S3_META_REQUEST_TYPE_PUT_OBJECT && meta_request->vtable->get_request_type &&
-           meta_request->vtable->get_request_type(request) == AWS_S3_REQUEST_TYPE_UPLOAD_PART;
-}
-
 void aws_s3_meta_request_send_request(struct aws_s3_meta_request *meta_request, struct aws_s3_connection *connection) {
     AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(connection);
@@ -875,7 +1051,7 @@ void aws_s3_meta_request_send_request(struct aws_s3_meta_request *meta_request,
         options.on_metrics = s_s3_meta_request_stream_metrics;
     }
     options.on_complete = s_s3_meta_request_stream_complete;
-    if (s_s3_request_is_upload_part(request)) {
+    if (request->request_type == AWS_S3_REQUEST_TYPE_UPLOAD_PART) {
         options.response_first_byte_timeout_ms = aws_atomic_load_int(&meta_request->client->upload_timeout_ms);
         request->upload_timeout_ms = (size_t)options.response_first_byte_timeout_ms;
     }
@@ -962,8 +1138,8 @@ static void s_get_part_response_headers_checksum_helper(
             aws_base64_compute_encoded_len(aws_get_digest_size_from_algorithm(i), &encoded_len);
             if (header_sum.len == encoded_len - 1) {
                 aws_byte_buf_init_copy_from_cursor(
-                    &connection->request->request_level_response_header_checksum, aws_default_allocator(), header_sum);
-                connection->request->request_level_running_response_sum = aws_checksum_new(aws_default_allocator(), i);
+                    &connection->request->request_level_response_header_checksum, meta_request->allocator, header_sum);
+                connection->request->request_level_running_response_sum = aws_checksum_new(meta_request->allocator, i);
             }
             break;
         }
@@ -990,9 +1166,9 @@ static void s_get_response_part_finish_checksum_helper(struct aws_s3_connection
         size_t encoded_checksum_len = 0;
         request->did_validate = true;
         aws_base64_compute_encoded_len(request->request_level_running_response_sum->digest_size, &encoded_checksum_len);
-        aws_byte_buf_init(&encoded_response_body_sum, aws_default_allocator(), encoded_checksum_len);
+        aws_byte_buf_init(&encoded_response_body_sum, request->allocator, encoded_checksum_len);
         aws_byte_buf_init(
-            &response_body_sum, aws_default_allocator(), request->request_level_running_response_sum->digest_size);
+            &response_body_sum, request->allocator, request->request_level_running_response_sum->digest_size);
         aws_checksum_finalize(request->request_level_running_response_sum, &response_body_sum, 0);
         struct aws_byte_cursor response_body_sum_cursor = aws_byte_cursor_from_buf(&response_body_sum);
         aws_base64_encode(&response_body_sum_cursor, &encoded_response_body_sum);
@@ -1061,7 +1237,7 @@ static int s_s3_meta_request_incoming_headers(
         s_s3_meta_request_error_code_from_response_status(request->send_data.response_status) == AWS_ERROR_SUCCESS;
 
     if (successful_response && meta_request->checksum_config.validate_response_checksum &&
-        request->request_tag == AWS_S3_AUTO_RANGE_GET_REQUEST_TYPE_PART) {
+        request->request_type == AWS_S3_REQUEST_TYPE_GET_OBJECT) {
         s_get_part_response_headers_checksum_helper(connection, meta_request, headers, headers_count);
     }
 
@@ -1084,6 +1260,14 @@ static int s_s3_meta_request_incoming_headers(
     return AWS_OP_SUCCESS;
 }
 
+/*
+ * Small helper to either do a static or dynamic append.
+ * TODO: something like this would be useful in common.
+ */
+static int s_response_body_append(bool is_dynamic, struct aws_byte_buf *buf, const struct aws_byte_cursor *data) {
+    return is_dynamic ? aws_byte_buf_append_dynamic(buf, data) : aws_byte_buf_append(buf, data);
+}
+
 static int s_s3_meta_request_incoming_body(
     struct aws_http_stream *stream,
     const struct aws_byte_cursor *data,
@@ -1117,17 +1301,19 @@ static int s_s3_meta_request_incoming_body(
     }
 
     if (request->send_data.response_body.capacity == 0) {
-        size_t buffer_size = s_dynamic_body_initial_buf_size;
-
-        if (request->part_size_response_body) {
-            buffer_size = meta_request->part_size;
+        if (request->has_part_size_response_body) {
+            AWS_FATAL_ASSERT(request->ticket);
+            request->send_data.response_body =
+                aws_s3_buffer_pool_acquire_buffer(request->meta_request->client->buffer_pool, request->ticket);
+        } else {
+            size_t buffer_size = s_dynamic_body_initial_buf_size;
+            aws_byte_buf_init(&request->send_data.response_body, meta_request->allocator, buffer_size);
         }
-
-        aws_byte_buf_init(&request->send_data.response_body, meta_request->allocator, buffer_size);
     }
 
-    if (aws_byte_buf_append_dynamic(&request->send_data.response_body, data)) {
-
+    /* Note: not having part sized response body means the buffer is dynamic and
+     * can grow. */
+    if (s_response_body_append(!request->has_part_size_response_body, &request->send_data.response_body, data)) {
         AWS_LOGF_ERROR(
             AWS_LS_S3_META_REQUEST,
             "id=%p: Request %p could not append to response body due to error %d (%s)",
@@ -1801,6 +1987,11 @@ void aws_s3_meta_request_result_setup(
             aws_byte_buf_init_copy(
                 result->error_response_body, meta_request->allocator, &failed_request->send_data.response_body);
         }
+
+        if (failed_request->operation_name != NULL) {
+            result->error_response_operation_name =
+                aws_string_new_from_string(meta_request->allocator, failed_request->operation_name);
+        }
     }
 
     result->response_status = response_status;
@@ -1820,6 +2011,8 @@ void aws_s3_meta_request_result_clean_up(
         aws_mem_release(meta_request->allocator, result->error_response_body);
     }
 
+    aws_string_destroy(result->error_response_operation_name);
+
     AWS_ZERO_STRUCT(*result);
 }
 
