@@ -521,8 +521,17 @@ int CBS_get_asn1_int64(CBS *cbs, int64_t *out) {
   uint8_t sign_extend[sizeof(int64_t)];
   memset(sign_extend, is_negative ? 0xff : 0, sizeof(sign_extend));
   for (size_t i = 0; i < len; i++) {
+// `data` is big-endian.
+// Values are always shifted toward the "little" end.
+#ifdef OPENSSL_BIG_ENDIAN
+    // Bytes are written starting at the highest index.
+    sign_extend[sizeof(sign_extend) - i - 1] = data[len - i - 1];
+#else
+    // Bytes are written starting at the lowest index.
     sign_extend[i] = data[len - i - 1];
+#endif
   }
+
   memcpy(out, sign_extend, sizeof(sign_extend));
   return 1;
 }
