@@ -22,14 +22,10 @@
 #ifndef __STDC_FORMAT_MACROS
 #define __STDC_FORMAT_MACROS
 #endif
-#include <inttypes.h>
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 #include <openssl/arm_arch.h>
 
+#include "cpu_aarch64.h"
 
 extern uint32_t OPENSSL_armcap_P;
 extern uint8_t OPENSSL_cpucap_initialized;
@@ -40,51 +36,6 @@ static uint64_t armv8_cpuid_probe(void) {
   return val;
 }
 
-// handle_cpu_env applies the value from |in| to the CPUID values in |out[0]|.
-// See the comment in |OPENSSL_cpuid_setup| about this.
-static void handle_cpu_env(uint32_t *out, const char *in) {
-  const int invert = in[0] == '~';
-  const int or = in[0] == '|';
-  const int skip_first_byte = invert || or;
-  const int hex = in[skip_first_byte] == '0' && in[skip_first_byte+1] == 'x';
-  uint32_t armcap = out[0];
-
-  int sscanf_result;
-  uint32_t v;
-  if (hex) {
-    sscanf_result = sscanf(in + skip_first_byte + 2, "%" PRIx32, &v);
-  } else {
-    sscanf_result = sscanf(in + skip_first_byte, "%" PRIu32, &v);
-  }
-
-  if (!sscanf_result) {
-    return;
-  }
-
-  // Detect if the user is trying to use the environment variable to set
-  // a capability that is _not_ available on the CPU:
-  // If getauxval() returned a non-zero hwcap in `armcap` (out)
-  // and a bit set in the requested `v` is not set in `armcap`,
-  // abort instead of crashing later.
-  // The case of invert cannot enable an unexisting capability;
-  // it can only disable an existing one.
-  if (!invert && armcap && (~armcap & v))
-  {
-    fprintf(stderr,
-            "Fatal Error: HW capability found: 0x%02X, but HW capability requested: 0x%02X.\n",
-            armcap, v);
-    exit(1);
-  }
-
-  if (invert) {
-    out[0] &= ~v;
-  } else if (or) {
-    out[0] |= v;
-  } else {
-    out[0] = v;
-  }
-}
-
 void OPENSSL_cpuid_setup(void) {
   unsigned long hwcap = getauxval(AT_HWCAP);
 
