@@ -100,6 +100,12 @@ int ECDH_compute_shared_secret(uint8_t *buf, size_t *buflen, const EC_POINT *pub
 #define EC_MAX_COMPRESSED (EC_MAX_BYTES + 1)
 #define EC_MAX_UNCOMPRESSED (2 * EC_MAX_BYTES + 1)
 
+#define EC_P521R1_FIELD_ELEM_BYTES 66
+#define EC_P384R1_FIELD_ELEM_BYTES 48
+#define EC_P256R1_FIELD_ELEM_BYTES 32
+#define EC_P224R1_FIELD_ELEM_BYTES 28
+#define EC_P256K1_FIELD_ELEM_BYTES 32
+
 OPENSSL_STATIC_ASSERT(EC_MAX_WORDS <= BN_SMALL_MAX_WORDS,
                       bn__small_functions_not_usable)
 
@@ -435,7 +441,7 @@ void ec_precomp_select(const EC_GROUP *group, EC_PRECOMP *out, BN_ULONG mask,
 
 // ec_cmp_x_coordinate compares the x (affine) coordinate of |p|, mod the group
 // order, with |r|. It returns one if the values match and zero if |p| is the
-// point at infinity of the values do not match.
+// point at infinity of the values do not match. |p| is treated as public.
 int ec_cmp_x_coordinate(const EC_GROUP *group, const EC_JACOBIAN *p,
                         const EC_SCALAR *r);
 
@@ -613,11 +619,10 @@ struct ec_group_st {
   // to avoid a reference cycle. Additionally, Z is guaranteed to be one, so X
   // and Y are suitable for use as an |EC_AFFINE|.
   EC_POINT *generator;
-  BIGNUM order;
 
   int curve_name;  // optional NID for named curve
 
-  BN_MONT_CTX *order_mont;  // data for ECDSA inverse
+  BN_MONT_CTX *order;
 
   // The following members are handled by the method functions,
   // even if they appear generic
@@ -634,13 +639,6 @@ struct ec_group_st {
   // otherwise.
   int field_greater_than_order;
 
-  // field_minus_order, if |field_greater_than_order| is true, is |field| minus
-  // |order| represented as an |EC_FELEM|. Otherwise, it is zero.
-  //
-  // Note: unlike |EC_FELEM|s used as intermediate values internal to the
-  // |EC_METHOD|, this value is not encoded in Montgomery form.
-  EC_FELEM field_minus_order;
-
   CRYPTO_refcount_t references;
 
   BN_MONT_CTX *mont;  // Montgomery structure.
